#! /home/upx/loaenv/bin/python3.12
# -*- coding: utf-8 -*-
"""
Created on 10octobre 2024
@author: Julien Gautier (LOA)
last modified 18 oct 2024

Dialog to RSAI motors rack via firebird database

"""

import time
from PyQt6 import QtCore
from PyQt6.QtWidgets import QMessageBox,QApplication
import  socket
from  PyQt6.QtCore import QUuid, QMutex
import sys, os 
import socket as _socket
import ast
import pathlib

p = pathlib.Path(__file__).parent
sepa = os.sep

fileconf = str(p) + sepa + "confServer.ini"
confServer = QtCore.QSettings(fileconf,QtCore.QSettings.Format.IniFormat)
server_host = str( confServer.value('MAIN'+'/server_host') )# 
serverPort =int(confServer.value('MAIN'+'/serverPort'))
clientSocket =_socket.socket(_socket.AF_INET,_socket.SOCK_STREAM)
#clientSocket.settimeout(5)

try :
    clientSocket.connect((server_host,serverPort))
    isconnected = True
    msg = 'clientid'
    clientSocket.sendall(msg.encode())
    id = clientSocket.recv(1024).decode()
    print('connected with id' ,id)
    

except :
    isconnected = False
    print('client not connected')

def listRack():
    cmdsend = " %s" %('listRack',)
    clientSocket.sendall((cmdsend).encode())
    listRack = clientSocket.recv(1024).decode()
    listRack = ast.literal_eval(listRack)
    return listRack

def nameEquipment(IP):
    cmd = 'nomRack'
    cmdsend = " %s, %s, %s " %(IP,1,cmd)
    clientSocket.sendall((cmdsend).encode())
    nameRack = clientSocket.recv(1024).decode().split()[0]
    return nameRack
    

mut = QMutex()

def closeConnection():
    # close connection
    clientSocket.close()

def listMotorName(IP):
    listMotor = []
    #print(IP)
    for i in range(0,14):
            cmd = 'name'
            cmdsend = " %s, %s, %s " %(IP,i+1,cmd)
            clientSocket.sendall((cmdsend).encode())
            name = clientSocket.recv(1024).decode() #.split()[0]
            listMotor.append(name)
    return listMotor

class MOTORRSAI():
    """
    MOTORRSAI(IpAdrress,NoMotor) 
    class is defined by Ipadress of the rack and axis number 
    """

    def __init__(self, IpAdrress,NoMotor,parent=None):
        
        self.IpAdress = IpAdrress
        self.NoMotor = NoMotor
        self.isconnected = isconnected
        self.clientSocket = clientSocket
        #self.mut = mut
        self.update()

    def update(self):
        '''update from the data base')
        '''

        self.name = self.getName()
        self.step = self.getStepValue()
        self.butPlus = self.getButLogPlusValue()
        self.butMoins = self.getButLogMoinsValue()

        self.refName=[]
        for i in range (0,6):
            r = self.getRefName(i)
            self.refName.append(r)
            
        self.refValue=[]
        for i in range (0,6):
            if self.step == 0:
                self.step = 1
            rr = self.getRefValue(i)/self.step
            self.refValue.append(rr)
            # time.sleep(0.01)

    def sendMessage(self,message=''):
        # print('message',self.isconnected)
        retour = '1'
        if self.isconnected is True:
            mut.lock()
            try:
                #print('lock')
                a = self.clientSocket.sendall(message.encode())
                retour = self.clientSocket.recv(1024).decode()
                self.isconnected = True
                retour =  retour.split()[0]
                
            except:
                self.isconnected = False
                self.clientSocket.close()
                print('error connection')

                retour = '1'
        if self.isconnected is False :    
            try: 
                #print('try again')
                self.clientSocket = _socket.socket(_socket.AF_INET,_socket.SOCK_STREAM)
                self.clientSocket.settimeout(2)
                self.clientSocket.connect((server_host,serverPort))
                self.isconnected = True
                retour = 1 
                #print('try again succes')
            except:
                #print('try again error')
                retour = '1' # avoid divide by zero 
                self.isconnected = False
                self.clientSocket.close()
                #print('socket close)')
        mut.unlock() 
            
        return retour
    
    def position(self):
        '''
        return motor postion
        '''
        cmd ='position'
        cmdsend = " %s, %s, %s " %(self.IpAdress,self.NoMotor,cmd)
        
        
        self._position = self.sendMessage(cmdsend) 
        try :
            self._position = float(self._position) 
        except :
            self._position = 1 
        return self._position
        
    
    def getName(self):
        '''
        get motor name
        '''
        cmd ='name'
        cmdsend = " %s, %s, %s " %(self.IpAdress,self.NoMotor,cmd)
        
        
        self._name = self.sendMessage(cmdsend)
        return self._name
    
    def setName(self,nom):
        '''
        set motor name
        '''
        cmd = 'setName'
        cmdsend = " %s, %s, %s,%s " %(self.IpAdress,self.NoMotor,cmd,nom)
        time.sleep(0.05)
        

    def getRefName(self,nRef) :
        '''
        get ref n° name
        '''
        cmd = 'ref' +str(nRef)+'Name'
        cmdsend = " %s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        
        dat = self.sendMessage(cmdsend)
        return dat
    
    def setRefName(self,nRef,name) :
        '''
        set ref n° name
        '''
        cmd ='setRefName'
        nRef = nRef+1
        nRef= str(nRef)
        name =str(name)
        cmdsend = " %s, %s, %s, %s, %s " %(self.IpAdress,self.NoMotor,cmd,name,nRef)
        dat = self.sendMessage(cmdsend)
        time.sleep(0.05)
        
    def getRefValue(self,nRef) :
        '''
        get value of the refereence position nRef
        '''
        nRef = nRef
        cmd = 'ref' +str(nRef)+'Pos'
        cmdsend = " %s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        dat = float(self.sendMessage(cmdsend))
        return dat
    
    def setRefValue(self,nReff,value) :
        '''
        set value of the refereence position nRef
        '''
        cmd = 'setRefPos'
        nReff = str(nReff+1)
        value = str(value)
        cmdsend = " %s, %s, %s, %s, %s" %(self.IpAdress,self.NoMotor,cmd,value,nReff)
        dat = self.sendMessage(cmdsend)
        

    def getStepValue(self):
        '''Valeur de 1 pas dans l'unites
        '''
        cmd = 'step'
        cmdsend = "%s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        
        
        dat = self.sendMessage(cmdsend)
        try :
            dat = float(dat)
        except :
            dat = 1 
        return dat

    def getButLogPlusValue(self):
        cmd = 'buteePos'
        cmdsend = "%s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        
        dat = self.sendMessage(cmdsend)
        try :
            dat = float(dat)
        except :
            dat = 1
        return dat
    
    def setButLogPlusValue(self,butPlus):
        """"""
    
    def getButLogMoinsValue(self):
        cmd = 'buteeNeg'
        cmdsend = "%s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        dat = self.sendMessage(cmdsend)
        try :
            dat = float(dat)
        except :
            dat = 1
        return dat
    
    def setButLogMoinsValue(self,butMoins):
        """"""

    def rmove(self,posrelatif,vitesse=1000):
        '''
        relative move of NoMotor of IpAdress
        posrelatif = position to move in step
        #to do faire self.curcwd
        '''
        cmd = 'rmove'
        pos = int(posrelatif)
        cmdsend = "%s, %s, %s,%s" %(self.IpAdress,self.NoMotor,cmd,pos)
        
        rec = self.sendMessage(cmdsend)
        if rec != 'ok':
            print('error cmd')
        

    def move(self,pos,vitesse=1000):
        '''absolue move of NoMotor  of IpAdress
        pos = position to move in step
        '''
        cmd = 'move'
        pos = int(pos)
        cmdsend =" %s, %s, %s,%s" %(self.IpAdress,self.NoMotor,cmd,pos)
        
        rec = self.sendMessage(cmdsend)
        if rec != 'ok':
            print('error cmd')

    def setzero(self):
        """
        setzero(self.moteurname):Set Zero
        """
        cmd = 'setzero'
    
        cmdsend = " %s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        rec = self.sendMessage(cmdsend)
        if rec != 'ok':
            print('error cmd')

    def stopMotor(self): # stop le moteur motor
        """ 
        stopMotor(motor): stop le moteur motor
        """
        cmd = 'stop'
    
        cmdsend =" %s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        
        rec = self.sendMessage(cmdsend)
        if rec != 'ok':
            print('error cmd')


    def etatMotor(self):
        '''
        read status of the motor
        '''
        cmd = 'etat'
        cmdsend = " %s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        
        dat = self.sendMessage(cmdsend)
        
        if self.isconnected is False :
            print('server not connected')
            dat = 'notconnected'
        return dat
        

    def getEquipementName(self):
        '''
        return the name of the equipement of which the motor is connected

        '''
        cmd = 'nomRack'
        cmdsend = " %s, %s, %s" %(self.IpAdress,self.NoMotor,cmd)
        
        dat = self.sendMessage(cmdsend)
        return dat
 


if __name__ == '__main__':
    a = MOTORRSAI('10.0.6.30',1)
    print(listRack())
    #a.setRefName(0,'test')
    #closeConnection()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 19 11:43:05 2018
@author: juliengautier

"""

from PyQt6.QtWidgets import QApplication, QVBoxLayout, QHBoxLayout, QWidget, QGridLayout
from PyQt6.QtWidgets import QLabel, QMainWindow, QMessageBox, QFileDialog
from PyQt6 import QtCore, QtGui 

from PyQt6.QtGui import QAction
from PyQt6.QtGui import QIcon
import sys
import time
import pyqtgraph as pg  # pyqtgraph biblio permettent l'affichage 
import numpy as np
import qdarkstyle  # pip install qdakstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
import pylab
import os
from scipy.ndimage.filters import gaussian_filter  # pour la reduction du bruit
# from scipy.interpolate import splrep, sproot # pour calcul fwhm et fit 
import pathlib
from scipy import ndimage


class WINPOINTING(QMainWindow):
    
    def __init__(self, parent=None, conf=None, name='VISU'):
        
        super().__init__()

        self.name = name
        self.parent = parent
        p = pathlib.Path(__file__)
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf
        
        sepa = os.sep
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.path = self.conf.value(self.name+"/path")
        self.isWinOpen = False
        self.setWindowTitle('Pointing')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.left = 100
        self.top = 30
        self.width = 800
        self.height = 800
        self.setGeometry(self.left, self.top, self.width, self.height)
        self.dimx = 1200
        self.dimy = 900
        self.bloqq = 1
        self.xec = int(self.conf.value(self.name+"/xec"))
        self.yec = int(self.conf.value(self.name+"/yec"))
        self.r1x = int(self.conf.value(self.name+"/r1x"))
        self.r1y = int(self.conf.value(self.name+"/r1y"))
        self.r2 = int(self.conf.value(self.name+"/r2x"))
        self.r2 = int(self.conf.value(self.name+"/r2y"))
        
        self.kE = 0  # variable pour la courbe E fct du nb shoot
        
        self.Xec = []
        self.Yec = []
        self.fwhmX = 100
        self.fwhmY = 100
        self.setWindowIcon(QIcon('./icons/LOA.png'))
        self.E = []
        
        # self.E = np.array([2,3,5])
        self.Xec = []
        self.Yec = []
        
        # Create x and y indices
        x = np.arange(0, self.dimx)
        y = np.arange(0, self.dimy)
        y, x = np.meshgrid(y, x)
    
        self.data = (40*np.random.rand(self.dimx, self.dimy)).round()
        
        self.setup()
        
        # self.Display(self.data)
        
    def setup(self):
        
        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.fileMenu = menubar.addMenu('&File')
        self.optionMenu = menubar.addMenu('&Options')
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.openAct.triggered.connect(self.OpenF)
        self.fileMenu.addAction(self.openAct)
        
        self.saveAct = QAction(QtGui.QIcon(self.icon+"disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.saveAct.triggered.connect(self.SaveF)
        self.fileMenu.addAction(self.saveAct)
        
        XYHLayout = QHBoxLayout()
        self.resetButton = QAction('Reset', self)
        self.resetButton.triggered.connect(self.Reset)
        self.optionMenu.addAction(self.resetButton)
        self.centerOfMass = QAction('Center of Mass', self)
        self.centerOfMass.setCheckable(True)
        self.centerOfMass.setChecked(False)
        self.optionMenu.addAction(self.centerOfMass)
        
        self.win1 = pg.PlotWidget()
        self.p1 = self.win1.plot(pen='w', symbol='t', symboleSize=2, clear=True, symbolPen='w', symbolBrush='w', name="XY")
        self.win1.setContentsMargins(0, 0, 0, 0)
        XYHLayout.addWidget(self.win1)
        
        self.win3 = pg.PlotWidget()
        self.p3 = self.win3.plot(pen='r', symbol='t', symboleSize=2, clear=True, symbolPen='r', symbolBrush='r', name="x")
        self.win3.setContentsMargins(0, 0, 0, 0)
        self.win3.setLabel('left', 'X')  # ,units='pixel')
        self.win3.setLabel('bottom', "Shot number")
        self.hLineMeanX = pg.InfiniteLine(angle=0, movable=False, pen=pg.mkPen('r', width=3, style=QtCore.Qt.DashLine))
        self.win3.addItem(self.hLineMeanX, ignoreBounds=True)
        
        labelMeanX = QLabel('<X>')
        self.meanXAff = QLabel()
        labelMeanX.setStyleSheet("color:red;font:14pt")
        
        labelStdX = QLabel('std X')
        labelStdX.setStyleSheet("color:red;font:14pt")
        self.stdXAff = QLabel()
        
        labelPVX = QLabel('PV X')
        labelPVX.setStyleSheet("color:red;font:14pt")
        self.PVXAff = QLabel()
        
        grid_layoutX = QGridLayout()
        grid_layoutX.addWidget(labelMeanX, 0, 0)
        grid_layoutX.addWidget(self.meanXAff, 0, 1)
        grid_layoutX.addWidget(labelStdX, 1, 0)
        grid_layoutX.addWidget(self.stdXAff, 1, 1)
        grid_layoutX.addWidget(labelPVX, 2, 0)
        grid_layoutX.addWidget(self.PVXAff, 2, 1)
        
        XHLayout = QHBoxLayout()
        XHLayout.addWidget(self.win3)
        
        dataxVLayout = QVBoxLayout()
        dataxVLayout.addStretch(2)
        dataxVLayout.addLayout(grid_layoutX)
        dataxVLayout.addStretch(2)
        XHLayout.addLayout(dataxVLayout)
        
        self.win4 = pg.PlotWidget()
        self.p4 = self.win4.plot(pen='g', symbol='t', symboleSize=2, clear=True, symbolPen='g', symbolBrush='g', name="y")
        self.win4.setLabel('left', 'Y')  # , units='pixel')
        self.win4.setLabel('bottom', "Shot number")
        self.hLineMeanY = pg.InfiniteLine(angle=0, movable=False, pen=pg.mkPen('g', width=3, style=QtCore.Qt.DashLine))
        self.win4.addItem(self.hLineMeanY, ignoreBounds=True)
        
        labelMeanY = QLabel('<Y>')
        self.meanYAff = QLabel()
        labelMeanY.setStyleSheet("color:green;font:14pt")
        
        labelStdY = QLabel('std Y')
        labelStdY.setStyleSheet("color:green;font:14pt")
        self.stdYAff = QLabel()
        labelPVY = QLabel('PV Y')
        labelPVY.setStyleSheet("color:green;font:14pt")
        self.PVYAff = QLabel()
        
        grid_layoutY = QGridLayout()
        grid_layoutY.addWidget(labelMeanY, 0, 0)
        grid_layoutY.addWidget(self.meanYAff, 0, 1)
        grid_layoutY.addWidget(labelStdY, 1, 0)
        grid_layoutY.addWidget(self.stdYAff, 1, 1)
        grid_layoutY.addWidget(labelPVY, 2, 0)
        grid_layoutY.addWidget(self.PVYAff, 2, 1)
        
        YHLayout = QHBoxLayout()
        YHLayout.addWidget(self.win4)
        
        datayVLayout = QVBoxLayout()
        datayVLayout.addStretch(2)
        datayVLayout.addLayout(grid_layoutY)
        datayVLayout.addStretch(2)
        YHLayout.addLayout(datayVLayout)
        
        vMainLayout = QVBoxLayout()
        
        # vMainLayout.addLayout(vbox1)
        vMainLayout.addLayout(XYHLayout)
        vMainLayout.addLayout(XHLayout)
        vMainLayout.addLayout(YHLayout)
        
        MainWidget = QWidget()
        
        MainWidget.setLayout(vMainLayout)
      
        self.setCentralWidget(MainWidget)
        self.setContentsMargins(1, 1, 1, 1)
        
        self.axeX1 = self.win1.getAxis('bottom')
        self.axeY1 = self.win1.getAxis('left')
        
        self.axeY3 = self.win3.getAxis('left')
        self.axeY4 = self.win4.getAxis('left')
        
        if self.parent is not None:
            self.parent.signalPointing.connect(self.Display)
        
    def Display(self, data, stepX=1, stepY=1):

        self.data = data
        self.dimx = self.data.shape[0]
        self.dimy = self.data.shape[1]
        self.stepX = stepX
        self.stepY = stepY
        dataF = gaussian_filter(self.data, 5)
        
        if self.centerOfMass.isChecked():
            (self.xec, self.yec) = ndimage.center_of_mass(dataF)
            self.label = 'com'

        else:
            (self.xec, self.yec) = pylab.unravel_index(dataF.argmax(), self.data.shape)
            self.label = 'max'
        self.xec = self.xec*stepX
        self.yec = self.yec*stepX
        
        self.Xec.append(self.xec)
        self.Yec.append(self.yec)
        
        Xmean = np.mean(self.Xec)
        self.meanXAff.setText('%.2f' % Xmean)
        stdX = np.std(self.Xec)
        self.stdXAff.setText('%.2f' % stdX)
        self.hLineMeanX.setPos(Xmean)
        Ymean = np.mean(self.Yec)
        self.meanYAff.setText('%.2f' % Ymean)
        stdV = np.std(self.Yec)
        self.stdYAff.setText('%.2f' % stdV)
        self.hLineMeanY.setPos(Ymean)
        
        XPV = max(self.Xec)-min(self.Xec)
        self.PVXAff.setText('%.2f' % XPV)
        
        YPV = max(self.Yec)-min(self.Yec)
        self.PVYAff.setText('%.2f' % YPV)
        
        self.p1.setData(y=self.Yec, x=self.Xec)
        self.p3.setData(self.Xec)  # ,pen='r',symbol='t',symboleSize=2,clear=True, symbolPen='r',symbolBrush='r',name="x")
        # self.p3.addItem(self.hLineMeanX, ignoreBounds=True)
        self.p4.setData(self.Yec)  # ,pen='g',symbol='t',symboleSize=2,clear=True,symbolPen='g',symbolBrush='g',name="y")
        # self.p4.addItem(self.hLineMeanY, ignoreBounds=True)
        
        if self.stepX != 1:
            self.axeX1.setLabel('X(um) ' + self.label)
            self.axeY3.setLabel('X(um) ' + self.label)
        else:
            self.axeX1.setLabel('X ' + self.label)
            self.axeY3.setLabel('X ' + self.label)
            
        if self.stepY != 1:
            self.axeY1.setLabel('Y(um) '+self.label)
            self.axeY4.setLabel('Y(um) '+self.label)
        else:
            self.axeY1.setLabel('Y '+self.label)
            self.axeY4.setLabel('Y '+self.label)
    
    def OpenF(self, fileOpen=False):

        if fileOpen is False:
            chemin = self.conf.value(self.name+"/path")
            fname = QFileDialog.getOpenFileName(self, "Open File", chemin, " 1D data (*.txt )")
            fichier = fname[0]
        else:
            fichier = str(fileOpen)
            
        ext = os.path.splitext(fichier)[1]
        
        if ext == '.txt':  # text file
            # self.cutData =np.loadtxt(str(fichier))
            aa = np.loadtxt(fichier, delimiter=" ", comments='#')
            self.Xec = aa[0]
            self.Yec = aa[1]
            
            self.p1.setData(y=self.Yec, x=self.Xec)
            self.p3.setData(self.Xec)
            self.p4.setData(self.Yec)
            Xmean = np.mean(self.Xec)
            self.meanXAff.setText('%.2f' % Xmean)
            stdX = np.std(self.Xec)
            self.stdXAff.setText('%.2f' % stdX)
            self.hLineMeanX.setPos(Xmean)
            Ymean = np.mean(self.Yec)
            self.meanYAff.setText('%.2f' % Ymean)
            stdV = np.std(self.Yec)
            self.stdYAff.setText('%.2f' % stdV)
            self.hLineMeanY.setPos(Ymean)
        
            XPV = max(self.Xec)-min(self.Xec)
            self.PVXAff.setText('%.2f' % XPV)
        
            YPV = max(self.Yec)-min(self.Yec)
            self.PVYAff.setText('%.2f' % YPV)
        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Critical)
            msg.setText("Wrong file format !")
            msg.setInformativeText("The format of the file must be : .txt ")
            msg.setWindowTitle("Warning ...")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec_()
    
    def SaveF(self):
        
        fname = QFileDialog.getSaveFileName(self, "Save data as txt", self.path)
        self.path = os.path.dirname(str(fname[0]))
        fichier = fname[0]
        # ext = os.path.splitext(fichier)[1]
        # print(ext)
        print(fichier, ' is saved')
        self.conf.setValue(self.name+"/path", self.path)
        time.sleep(0.1)
        a=np.reshape(np.array(self.Xec),(len(self.Xec),1))
        b=np.reshape(np.array(self.Yec),(len(self.Yec),1))
        mat=np.concatenate((a,b),axis=1)
        np.savetxt(str(fichier)+'.txt',mat.T)#,header="StepX:"+str(self.stepX)+"StepY:"+str(self.stepY))
        
    def Reset(self):
        print('reset)')
        self.Xec = []
        self.Yec = []
        
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        self.E = []
        self.Xec = []
        self.Yec = []
        time.sleep(0.1)
        event.accept()
     

if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = WINPOINTING(name='VISU')
    e.show()
    appli.exec_()



from PyQt6 import QtWidgets, QtCore
import sys
from PyQt6.QtWidgets import QApplication
import pathlib
from PyQt6.QtGui import QIcon
import qdarkstyle
import os
import pyqtgraph as pg
import numpy as np

class InputE(QtWidgets.QWidget):
    closeEventVar = QtCore.pyqtSignal(bool)

    def __init__(self,parent=None):

        super().__init__()

        p = pathlib.Path(__file__)
        sepa = os.sep
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.isWinOpen = False
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.defvalfile = QtCore.QSettings(str(pathlib.Path(__file__).parent / 'default_values.ini'), QtCore.QSettings.Format.IniFormat)
        self.buttonSelected = False
        self.buttonSelectedBack = False
        self.hminBg = 0
        self.hmaxBg = 1
        self.setup()
        self.actionButton()
        self.setWindowTitle('Spectro input')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.parent=parent

    def setup(self):
        # definition of all button   
        
        layout = QtWidgets.QVBoxLayout(self)
        
        # Create layout for dimension with four input entries
        dimension_layout = QtWidgets.QHBoxLayout()

        dimension_layout.addWidget(QtWidgets.QLabel('Dimensions :'))
        
        self.wmin = QtWidgets.QSpinBox()
        #self.wmin.setPlaceholderText('Left limit')
        self.wmin.setMaximum(10000)
        
        self.wmin.setValue(int(self.defvalfile.value("/"+"/wmin")))
        dimension_layout.addWidget(self.wmin)

        self.wmax = QtWidgets.QSpinBox()
        #self.wmax.setPlaceholderText('Right limit')
        self.wmax.setMaximum(10000)
        self.wmax.setValue(int(self.defvalfile.value("/"+"/wmax")))
        dimension_layout.addWidget(self.wmax)

        self.hmin = QtWidgets.QSpinBox()
        self.hmin.setMaximum(10000)
        self.hmin.setValue(int(self.defvalfile.value("/"+"/hmin")))
        dimension_layout.addWidget(self.hmin)

        self.hmax = QtWidgets.QSpinBox()
        self.hmax.setMaximum(10000)
        self.hmax.setValue(int(self.defvalfile.value("/"+"/hmax")))
        dimension_layout.addWidget(self.hmax)
        
        self.selectButton=QtWidgets.QToolButton()
        self.selectButton.setText('Select')
        dimension_layout.addWidget(self.selectButton)

        layout.addLayout(dimension_layout)
        ppmm_layout = QtWidgets.QHBoxLayout()
        ppmm_layout.addWidget(QtWidgets.QLabel('Pixel per millimeter :'))
        self.ppmm = QtWidgets.QDoubleSpinBox()
        self.ppmm.setMaximum(10000)
        ppmm_layout.addWidget(self.ppmm)
        self.ppmm.setValue(float(self.defvalfile.value("/"+"/ppmm")))
        layout.addLayout(ppmm_layout)

        # Create layout for pixel position of high energy edge s0
        pps0_layout = QtWidgets.QHBoxLayout()
        pps0_layout.addWidget(QtWidgets.QLabel('Pixel position of zero Lanex :'))
        self.pps0 = QtWidgets.QDoubleSpinBox()
        pps0_layout.addWidget(self.pps0)
        self.pps0.setMaximum(10000)
        self.pps0.setValue(float(self.defvalfile.value("/"+"/pps0")))
        layout.addLayout(pps0_layout)

        # Create layout for source screen distance
        ssd_layout = QtWidgets.QHBoxLayout()
        ssd_layout.addWidget(QtWidgets.QLabel('Source screen distance :'))
        self.ssd = QtWidgets.QDoubleSpinBox()
        self.ssd.setMaximum(10000)
        ssd_layout.addWidget(self.ssd)
        self.ssd.setValue(float(self.defvalfile.value("/"+"/ssd")))
        layout.addLayout(ssd_layout)
        
        # Create layout for median filter strength with increase button
        medfilt_layout = QtWidgets.QHBoxLayout()
        medfilt_layout.addWidget(QtWidgets.QLabel('Median Filter Strength :'))
        self.medfilt = QtWidgets.QSpinBox()
        self.medfilt.setMaximum(100)
        medfilt_layout.addWidget(self.medfilt)
        self.medfilt.setValue((int(self.defvalfile.value("/"+"/medfilt"))))
        layout.addLayout(medfilt_layout)

        # Create layout for number of energy points
        npoints_layout = QtWidgets.QHBoxLayout()
        npoints_layout.addWidget(QtWidgets.QLabel('Number of energy points :'))
        self.npoints = QtWidgets.QDoubleSpinBox()
        self.npoints.setMaximum(10000)
        npoints_layout.addWidget(self.npoints)
        self.npoints.setValue((int(self.defvalfile.value("/"+"/npoints"))))
        layout.addLayout(npoints_layout)
        count_layout = QtWidgets.QHBoxLayout()
        count_layout.addWidget(QtWidgets.QLabel('Charge/pixel :'))
        self.count = QtWidgets.QDoubleSpinBox()
        self.count.setMaximum(10000)
        count_layout.addWidget(self.count)
        self.count.setValue((int(self.defvalfile.value("/"+"/count"))))
        layout.addLayout(count_layout)

        # Create Select dsdE file button
        self.select_dsde_button = QtWidgets.QPushButton('Select dsdE file', self)
        layout.addWidget(self.select_dsde_button)

        # Create a label to display the selected dsdE file name
        self.selected_dsde_label = QtWidgets.QLineEdit(self)
        layout.addWidget(self.selected_dsde_label)
        self.dsde_name = (str(self.defvalfile.value("/"+"/dsde_name")))
        self.selected_dsde_label.setText(self.dsde_name)
        # Read ds_dE data from file
        try:
         self.slist, self.elist, self.dsdelist,self.thetalist,self.dlist = self.readfile(self.dsde_name)
        except:
            self.slist, self.elist, self.dsdelist,self.thetalist,self.dlist=0,0,0,0,0
            print('error :  electrons file calibration not defined' )

        # Create label to display the warning message if no file has been selected
        self.warning_label = QtWidgets.QLabel('', self)
        self.warning_label.setStyleSheet('color: red')
        layout.addWidget(self.warning_label)
        
        self.backgroudButton = QtWidgets.QPushButton('Select ROI background', self)
        layout.addWidget(self.backgroudButton)

        self.setLayout(layout)
        # rect for Background
        self.rectSelectBack = pg.RectROI([100, 10], [4*100, 15],
                                            pen='r')

    def actionButton(self):
        self.wmin.editingFinished.connect(self.set_default)
        self.wmax.editingFinished.connect(self.set_default)
        self.hmin.editingFinished.connect(self.set_default)
        self.hmax.editingFinished.connect(self.set_default)
        self.medfilt.editingFinished.connect(self.set_default)
        self.npoints.editingFinished.connect(self.set_default)
        self.ppmm.editingFinished.connect(self.set_default)
        self.pps0.editingFinished.connect(self.set_default)
        self.ssd.editingFinished.connect(self.set_default)
        self.count.editingFinished.connect(self.set_default)
        self.selected_dsde_label.textChanged.connect(self.dsdseChange)
        self.select_dsde_button.clicked.connect(self.select_dsde)
        self.backgroudButton.clicked.connect(self.selectBackground)
        self.rectSelectBack.sigRegionChangeFinished.connect(self.changeDimbg)

    def select_dsde(self):
        # Pick dsdE file 
        dsde_dialog = QtWidgets.QFileDialog()
        self.dsde_name, _ = dsde_dialog.getOpenFileName()
        
        # Update the label text to display the selected file name
        if self.dsde_name:
            self.selected_dsde_label.setText(str(self.dsde_name))
    
    def set_default(self):
        # save values in the default ini file
        self.defvalfile.setValue("/"+"/wmin",self.wmin.value())
        self.defvalfile.setValue("/"+"/wmax",self.wmax.value())
        self.defvalfile.setValue("/"+"/hmin",self.hmin.value())
        self.defvalfile.setValue("/"+"/hmax",self.hmax.value())
        self.defvalfile.setValue("/"+"/medfilt",self.medfilt.value())
        self.defvalfile.setValue("/"+"/npoints",self.npoints.value())
        self.defvalfile.setValue("/"+"/ppmm",self.ppmm.value())
        self.defvalfile.setValue("/"+"/pps0",self.pps0.value())
        self.defvalfile.setValue("/"+"/ssd",self.ssd.value())
        self.defvalfile.setValue("/"+"/count",self.count.value())

    def dsdseChange(self):
        self.defvalfile.setValue("/"+"/dsde_name",self.dsde_name)
        # Read ds_dE data from file
        self.slist, self.elist, self.dsdelist,self.thetalist,self.dlist = self.readfile(self.dsde_name)
        
    def selectBackground(self):
        if self.parent is not None:
            if self.buttonSelectedBack is False: 
                self.rectSelectBack.setPos([self.parent.energy_min,0])
                self.rectSelectBack.setSize([self.parent.energy_max-self.parent.energy_min,3])
                self.parent.p1.addItem(self.rectSelectBack)
                self.buttonSelectedBack = True
                self.parent.checkBoxBg.setChecked(True)
            else:
                self.buttonSelectedBack = False
                self.parent.p1.removeItem(self.rectSelectBack)
            
    def changeDimbg(self):
        self.wminBg = 0 #  (int(self.rectSelectSpectro.pos()[0]))
        self.wmaxBg = self.parent.dimx #  (int(self.rectSelectSpectro.pos()[0] + self.rectSelectSpectro.size()[0] ))
        self.hminBg = int((self.rectSelectBack.pos()[1]-self.parent.transY*self.parent.scaleY)/self.parent.scaleY)
        self.hmaxBg = int((self.rectSelectBack.size()[1]+self.rectSelectBack.pos()[1] -  self.parent.transY*self.parent.scaleY)/self.parent.scaleY)

        if self.parent is not None:
            self.parent.SpectroChanged()

    def readfile(self,filename):
        elist,dsdelist,slist,thetalist,dlist=np.loadtxt(str(filename),unpack=True)
        # elist list de energie
        # slist list des postion sur le lanex
        # dsdelist derive position/energie
        # thetalist liste angle sur lannex
        # dlist distance parcourue par les electrons
        
        if elist[1] < elist[0]:
            slist = slist[::-1] # reversed slist
            elist = elist[::-1] # reversed elist
            dsdelist = dsdelist[::-1] # reversed elist 
            thetalist = thetalist[::-1]
            dlist =dlist[::-1]
        
        return slist, elist,dsdelist,thetalist,dlist

    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        self.set_default()
        if self.parent is not None :
            if self.parent.parent is not None:
            # closing rectangular selection of size on visu 
                if self.buttonSelected is True: 
                    self.parent.parent.p1.removeItem(self.parent.rectSelectSpectro)
        if self.parent is not None: # supress back roi on spectrowindow
            if self.buttonSelectedBack is True:
                self.parent.p1.removeItem(self.rectSelectBack)

        self.closeEventVar.emit(True)
        
if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = InputE()
    e.show()
    appli.exec_()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 10 09:44:07 2019
Window for Measurement
@author: juliengautier
modified 2019/08/13 : add motors RSAI position and zoom windows
modified 2023/07/04 add user function
"""


import qdarkstyle 
from PyQt6 import QtCore, QtGui
from PyQt6.QtWidgets import QApplication, QVBoxLayout, QHBoxLayout, QMainWindow, QHeaderView
from PyQt6.QtWidgets import QWidget, QTableWidget, QTableWidgetItem, QAbstractItemView, QComboBox, QInputDialog
from PyQt6.QtGui import QAction
from PyQt6.QtGui import QIcon
from PyQt6.QtCore import Qt
from visu.WinCut import GRAPHCUT
from visu.winZoom import ZOOM
import pathlib
import numpy as np
import sys
import time
import os
from scipy import ndimage



class MEAS(QMainWindow):
    
    signalPlot = QtCore.pyqtSignal(object)
    
    def __init__(self, parent=None, conf=None, name='VISU', confMot=None, **kwds):
        
        super().__init__()

        self.parent = parent
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        p = pathlib.Path(__file__)
        sepa = os.sep
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.IniFormat)
        else:
            self.conf = conf
        self.confMot = confMot   # le Qsetting des moteurs obsolete 
        self.name = name
        self.ThresholdState = False
        self.symbol = False
        
        if 'motRSAI' in kwds :
            self.motRSAI = kwds["motRSAI"]
            if self.motRSAI is True:
                import visu.moteurRSAISERVER as RSAI
                self.RSAI = RSAI
                self.listRack = self.RSAI.listRack()
                self.IPadress = self.listRack[0]
                self.rackName = []
                self.listMotorName = self.RSAI.listMotorName(self.IPadress)
                print('RSAI motor connected to python server')  
        else : 
            self.motRSAI = False
        
        if 'motA2V' in kwds :
            self.motA2V = kwds["motA2V"]
            if self.motA2V is True:
                # import importlib.utils
                # module_name = 'moteurA2V'
                sys.path.append('C:/Users/UPX/Desktop/python/camera')
                import moteurA2V as A2V
                self.A2V = A2V
                self.listRack = list()
                
                self.rackName = []
                self.listMotorName = self.A2V.listMotorName
                self.listMotor = self.A2V.listMotor
                print('TMCL motor connected to database')  
        else : 
            self.motA2V = False


        self.indexUnit = 1 # micron
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.isWinOpen = False
        self.setup()
        
        self.setWindowTitle('MEASUREMENTS')
        self.shoot = 0
        self.nomFichier = ' '
        self.TableSauv = ['file,Max,Min,x Max,y max,Sum,Mean,Size,x c.mass,y c.mass, user1']
        
        self.path = self.conf.value(self.name+"/path")
        self.winCoupeMax = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeMin = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeXmax = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeYmax = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeSum = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeMean = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeXcmass = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeYcmass = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeSumThreshold = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.winCoupeUser1 = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None,lastColored=True)
        self.signalTrans = dict()
        
        self.Maxx = []
        self.Minn = []
        self.Summ = []
        self.Mean = []
        self.Xmax = []
        self.Ymax = []
        self.Xcmass = []
        self.Ycmass = []
        self.labelsVert = []
        self.posMotor = []
        self.SummThre = []
        self.USER1 = []
        self.winZoomMax = ZOOM()
        self.winZoomSum = ZOOM()
        self.winZoomMean = ZOOM()
        self.winZoomXmax = ZOOM()
        self.winZoomYmax = ZOOM()
        self.winZoomCxmax = ZOOM()
        self.winZoomCymax = ZOOM()
        self.winZoomSumThreshold = ZOOM()
        self.winZoomUser1 = ZOOM()
        self.maxx = 0
        self.summ = 0
        self.moy = 0
        self.user1 = 0
        self.label = 'Shoot'
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setGeometry(100, 300, 1000, 300)
        if self.motRSAI is True or self.motA2V is True:
            self.stepmotor = 1
            self.unit()

    def setFile(self, file):
        self.nomFichier = file
        
    def setup(self):
            
        vLayout = QVBoxLayout()
        hLayout1 = QHBoxLayout()
        # self.toolBar  = self.addToolBar('tools')
        # self.setStyleSheet("{background-color: black}")
        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.fileMenu = menubar.addMenu('&File')
        self.PlotMenu = menubar.addMenu('&Plot')
        self.ZoomMenu = menubar.addMenu('&Zoom')
        
        self.ThresholdAct = QAction('Threshold', self)
        self.ThresholdMenu = menubar.addAction(self.ThresholdAct)
        self.ThresholdAct.triggered.connect(self.Threshold)
        
        self.setContentsMargins(0, 0, 0, 0)
       
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.openAct.triggered.connect(self.openF)
        
        self.fileMenu.addAction(self.openAct)
        
        self.saveAct = QAction(QtGui.QIcon(self.icon+"disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.saveAct.triggered.connect(self.saveF)
        
        self.fileMenu.addAction(self.saveAct)
        
        self.PlotMenu.addAction('max', self.PlotMAX)
        self.PlotMenu.addAction('min', self.PlotMIN)
        self.PlotMenu.addAction('x max', self.PlotXMAX)
        self.PlotMenu.addAction('y max', self.PlotYMAX)
        self.PlotMenu.addAction('Sum', self.PlotSUM)
        self.PlotMenu.addAction('Mean', self.PlotMEAN)
        self.PlotMenu.addAction('x center mass', self.PlotXCMASS)
        self.PlotMenu.addAction('y center mass', self.PlotYCMASS)
        self.PlotMenu.addAction('User 1', self.PlotUSER1)

        self.ZoomMenu.addAction('max', self.ZoomMAX)
        self.ZoomMenu.addAction('Sum', self.ZoomSUM)
        self.ZoomMenu.addAction('Mean', self.ZoomMEAN)
        self.ZoomMenu.addAction('X max', self.ZoomXmax) 
        self.ZoomMenu.addAction('Y max', self.ZoomYmax)
        self.ZoomMenu.addAction(' X c.of.m', self.ZoomCxmax)
        self.ZoomMenu.addAction(' Y c.of.m', self.ZoomCymax)
        self.ZoomMenu.addAction(' User1', self.ZoomUser1)
        
        self.But_reset = QAction('Reset', self)
        # self.PlotMenu.addAction(self.But_reset)
        menubar.addAction(self.But_reset)
        
        hLayout2 = QHBoxLayout()
        self.table = QTableWidget()
        hLayout2.addWidget(self.table)
        
        self.table.setColumnCount(12)
        # self.table.setRowCount(10)
       
        self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'date'))
        header = self.table.horizontalHeader()
        header.setDefaultAlignment(Qt.AlignmentFlag.AlignHCenter)
        header.setDefaultAlignment(Qt.AlignmentFlag.AlignVCenter)
        header.setSectionResizeMode(QHeaderView.ResizeMode.ResizeToContents)

        if self.motRSAI is True or self.motA2V is True:
            self.motorNameBox = QComboBox()
            self.motorNameBox.addItem('Choose a Motor')
            self.unitBouton = QComboBox()
            self.unitBouton.addItem('Step')
            self.unitBouton.addItem('um')
            self.unitBouton.addItem('mm')
            self.unitBouton.addItem('ps')
            self.unitBouton.addItem('°')
            self.unitBouton.setMaximumWidth(100)
            self.unitBouton.setMinimumWidth(100)
            self.unitBouton.setCurrentIndex(self.indexUnit)
            hLayout1.addWidget(self.unitBouton)
            self.unitBouton.currentIndexChanged.connect(self.unit) 
            self.rackChoise = QComboBox()

            for rack in self.listRack: #
                self.rackChoise.addItem( str(self.RSAI.nameEquipment(rack))+ '  (' + rack +')')
            hLayout1.addWidget(self.rackChoise)
            self.motorNameBox.addItems(self.listMotorName)
            hLayout1.addWidget(self.motorNameBox)
            self.table.setColumnCount(13)   
            self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'Motor', 'date'))
            
            self.rackChoise.currentIndexChanged.connect(self.ChangeIPRack)
            self.motorNameBox.currentIndexChanged.connect(self.motorChange)
            
             
        self.table.horizontalHeader().setVisible(True)
        self.table.setAlternatingRowColors(True)
        self.table.resizeColumnsToContents()
        self.table.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)  # no modifiableEditTriggers=QAbstractItemView.editTriggers(QAbstractItemView.NoEditTriggers)
        
        vLayout.addLayout(hLayout1)
        vLayout.addLayout(hLayout2)
        
        MainWidget = QWidget()
        MainWidget.setLayout(vLayout)
        self.setCentralWidget(MainWidget)
        self.setContentsMargins(1, 1, 1, 1)
        
        if self.parent is not None:
            self.parent.signalMeas.connect(self.Display)
            
        # if self.parent is not None:
        #     self.parent.newMesurment.connect(self.Display)
        self.But_reset.triggered.connect(self.Reset)
        
    def Reset(self):
        self.shoot = 0
        self.table.setRowCount(0)
        self.Maxx = []
        self.Minn = []
        self.Summ = []
        self.Mean = []
        self.Xmax = []
        self.Ymax = []
        self.Xcmass = []
        self.Ycmass = []
        self.labelsVert = []
        self.posMotor = []
        self.SummThre = []
        self.USER = []

    def saveF(self):
       
        fname = QtGui.QFileDialog.getSaveFileName(self, "Save Measurements as txt file", self.path)
        self.path = os.path.dirname(str(fname[0]))
        f = open(str(fname[0])+'.txt', 'w')
        f.write("\n".join(self.TableSauv))
        f.close()
        
    def openF(self):
        '''
        to do
        '''
        print('open not done yet')
    
    def ZoomMAX(self):
        self.open_widget(self.winZoomMax)
        self.winZoomMax.SetTITLE('MAX')
        self.winZoomMax.setZoom(self.maxx)
        
    def ZoomSUM(self):
        self.open_widget(self.winZoomSum)
        self.winZoomSum.SetTITLE('Sum')
        self.winZoomSum.setZoom(self.summ)
        
    def ZoomMEAN(self):
        self.open_widget(self.winZoomMean)
        self.winZoomMean.SetTITLE('Mean')
        self.winZoomMean.setZoom(self.moy)    
   
    def ZoomXmax(self):
        self.open_widget(self.winZoomXmax)
        self.winZoomXmax.SetTITLE('x max')
        self.winZoomXmax.setZoom(self.xmax)
        
    def ZoomYmax(self):
        self.open_widget(self.winZoomYmax)
        self.winZoomYmax.SetTITLE('y max')
        self.winZoomYmax.setZoom(self.ymax) 
    
    def ZoomCxmax(self):
        self.open_widget(self.winZoomCymax)
        self.winZoomCxmax.SetTITLE('x center of mass')
        self.winZoomCxmax.setZoom(self.xcmass)   
    
    def ZoomCymax(self):
        self.open_widget(self.winZoomCxmax)
        self.winZoomCymax.SetTITLE('y center of mass')
        self.winZoomCymax.setZoom(self.ycmass)
        
    def ZoomSUMThreshold(self):
        self.open_widget(self.winZoomSumThreshold)
        
        self.winZoomSumThreshold.SetTITLE('Sum threshold')
        self.winZoomSumThreshold.setZoom(self.summThre) 
    
    def ZoomUser1(self):
        self.open_widget(self.winZoomUser1)
        
        self.winZoomUser1.SetTITLE('User 1')
        self.winZoomUser1.setZoom(self.user1)

    def PlotMAX(self):
        self.open_widget(self.winCoupeMax)
        self.winCoupeMax.SetTITLE('Plot Max')
        self.signalTrans['data'] = self.Maxx
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        # self.winCoupeMax.PLOT(self.Maxx,axis = self.posMotor, label = self.label)
    
    def PlotMIN(self):
        self.open_widget(self.winCoupeMin)
        self.winCoupeMin.SetTITLE('Plot Min')
        self.signalTrans['data'] = self.Minn
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        # self.winCoupeMin.PLOT(self.Minn,axis = self.posMotor,label = self.label)
        
    def PlotXMAX(self):
        self.open_widget(self.winCoupeXmax)
        self.winCoupeXmax.SetTITLE('Plot  X MAX')
        self.signalTrans['data'] = self.Xmax
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        
        # self.winCoupeXmax.PLOT(self.Xmax,axis = self.posMotor,label = self.label)
    
    def PlotYMAX(self):
        self.open_widget(self.winCoupeYmax)
        self.winCoupeYmax.SetTITLE('Plot  Y MAX')
        self.signalTrans['data'] = self.Ymax
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        # self.winCoupeYmax.PLOT(self.Ymax,axis = self.posMotor,label = self.label)
     
    def PlotSUM(self):
        self.open_widget(self.winCoupeSum)
        self.winCoupeSum.SetTITLE('Plot Sum')
        self.signalTrans['data'] = self.Summ
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        # self.winCoupeSum.PLOT(self.Summ,axis = self.posMotor,label = self.label)
    
    def PlotMEAN(self):
        self.open_widget(self.winCoupeMean)
        self.winCoupeMean.SetTITLE('Plot Mean')
        self.signalTrans['data'] = self.Mean
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        # self.winCoupeMean.PLOT(self.Mean,axis = self.posMotor,label = self.label)
        
    def PlotXCMASS(self):
        self.open_widget(self.winCoupeXcmass)
        self.winCoupeXcmass.SetTITLE('Plot x center of mass')
        self.signalTrans['data'] = self.Xcmass
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        # self.winCoupeXcmass.PLOT(self.Xcmass,axis = self.posMotor,label = self.label)
    
    def PlotYCMASS(self):
        self.open_widget(self.winCoupeYcmass)
        self.winCoupeYcmass.SetTITLE('Plot Y center of mass')
        self.signalTrans['data'] = self.Ycmass
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)
        # self.winCoupeYcmass.PLOT(self.Xcmass,axis = self.posMotor,label = self.label)    
    
    def PlotSUMTHRESHOLD(self):
        self.open_widget(self.winCoupeSumThreshold)
        self.winCoupeSumThreshold.SetTITLE('Plot Sum with Threshold')
        # print('fct',self.SummThre,self.posMotor)
        self.signalTrans['data'] = self.SummThre
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
       
        self.signalPlot.emit(self.signalTrans)
    
    def Threshold(self):
        
        threshold, ok = QInputDialog.getInt(self, 'Threshold Filter ', 'Enter thresold value')
        if ok:
            self.ThresholdState = True
            self.threshold = threshold
            self.Reset()
            self.PlotMenu.addAction('Sum Threshold', self.PlotSUMTHRESHOLD)
            self.ZoomMenu.addAction(' Sum  with Threshold', self.ZoomSUMThreshold)
            self.Display(self.data)
        else:
            self.ThresholdState = False
            self.PlotMenu.removeAction('Sum Threshold', self.PlotSUMTHRESHOLD)
            self.ZoomMenu.removeAction(' Sum  with Threshold', self.ZoomSUMThreshold)

    def PlotUSER1(self):
        self.openwidget(self.winCoupeUser1)
        self.winCoupeUser1.SetTITLE('User1')
        self.signalTrans['data'] = self.USER1
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        
        self.signalPlot.emit(self.signalTrans)

    def Display(self, data):
        self.data = data[0]
        self.transx=data[1]
        self.transy=data[2]
        self.scalex=data[3]
        self.scaley=data[4]
        
        #print(self.transx,self.scaley)
        self.maxx = round(self.data.max(), 3)
        self.minn = round(self.data.min(), 3)
        self.summ = round(self.data.sum(), 3)
        self.moy = round(self.data.mean(), 3)
        self.user1 = round(self.FctUser1(), 3)
        self.date = time.strftime("%Y_%m_%d_%H_%M_%S")
        
        (self.xmax, self.ymax) = np.unravel_index(self.data.argmax(), self.data.shape)
        self.xmax = (self.xmax + self.transx) * self.scalex 
        self.ymax = (self.ymax  + self.transy) * self.scaley 
        # print(self.maxx,data[int(self.xmax),int(self.ymax)])
        (self.xcmass, self.ycmass) = ndimage.center_of_mass(self.data)
        self.xcmass = (round(self.xcmass, 3) + self.transx) *self.scalex
        self.ycmass = (round(self.ycmass, 3) + self.transy) *self.scaley
        
        self.xs = self.data.shape[0]
        self.ys = self.data.shape[1]
        self.table.setRowCount(self.shoot+1)
        self.table.setItem(self.shoot, 0, QTableWidgetItem(str(self.nomFichier)))
        self.table.setItem(self.shoot, 1, QTableWidgetItem(str(self.maxx)))
        self.table.setItem(self.shoot, 2, QTableWidgetItem(str(self.minn)))
        self.table.setItem(self.shoot, 3, QTableWidgetItem(str(self.xmax)))
        self.table.setItem(self.shoot, 4, QTableWidgetItem(str(self.ymax)))
        self.table.setItem(self.shoot, 5, QTableWidgetItem("{:.3e}".format(self.summ)))
        self.table.setItem(self.shoot, 6, QTableWidgetItem(str(self.moy)))
        self.table.setItem(self.shoot, 7, QTableWidgetItem((str(self.xs) + '*' + str(self.ys))))
        self.table.setItem(self.shoot, 8, QTableWidgetItem(str(self.xcmass)))
        self.table.setItem(self.shoot, 9, QTableWidgetItem(str(self.ycmass)))
        self.table.setItem(self.shoot, 10, QTableWidgetItem(str(self.user1)))

        if self.motRSAI is True or self.motA2V is True :
            if self.motorNameBox.currentIndex() ==0 :
                Posi = self.shoot
                self.label = 'Shoot'
            else:
                Posi = round((self.MOT.position())/self.unitChange,2)
                
                self.label = self.MOT.name + '  ( '+self.unitName+' )'
        
        else:
            Posi = self.shoot
            self.label = 'Shoot'
            
        self.posMotor.append(Posi)    
        self.table.resizeColumnsToContents()
        self.labelsVert.append('%s' % self.shoot)
        
        if self.ThresholdState is True:
            dataCor = np.where(self.data < self.threshold, 0, data)
            self.summThre = round(dataCor.sum(), 3)
            self.SummThre.append(self.summThre)
            self.TableSauv.append('%s,%.1f,%.1f,%i,%i,%.1f,%.3f,%.2f,%.2f,%.2f, %.2f,%.2f,%.2f,%.2f,%s' % (self.nomFichier, self.maxx, self.minn, self.xmax, self.ymax, self.summ, self.moy, self.xs, self.ys, self.xcmass, self.ycmass, Posi, self.summThre, self.user1, self.date))
            
            if self.motRSAI is True or self.motA2V is True:
                self.table.setColumnCount(14)
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum Mean', 'Size', 'x c.mass', 'y c.mass', 'Sum Thr', ' Motor', 'date'))
                self.table.setItem(self.shoot, 12, QTableWidgetItem(str(Posi)))
                self.table.setItem(self.shoot, 13, QTableWidgetItem(str(self.date)))
            else:
                self.table.setColumnCount(13)
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'Sum Thr', 'user1', 'date'))
                self.TableSauv = ['file,Max,Min,x Max,y max,Sum,Mean,Size,x c.mass, y c.mass,SumCorr,date']
                self.table.setItem(self.shoot, 11, QTableWidgetItem(str(self.date)))
            self.table.setItem(self.shoot, 10, QTableWidgetItem("{:.3e}".format(self.summThre)))
            
        else:
            self.TableSauv.append('%s,%.1f,%.1f,%i,%i,%.1f,%.3f,%.2f,%.2f,%.2f, %.2f,%.2f,%.2f,%s' % (self.nomFichier, self.maxx, self.minn, self.xmax, self.ymax, self.summ, self.moy, self.xs, self.ys, self.xcmass, self.ycmass, self.user1, Posi, self.date))
            
            if self.motRSAI is True or self.motA2V is True:
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'Motor', 'date'))
                self.table.setColumnCount(13)
                self.table.setItem(self.shoot, 11, QTableWidgetItem(str(Posi)))
                self.table.setItem(self.shoot, 12, QTableWidgetItem(str(self.date)))
            else:
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'date'))
                self.table.setColumnCount(12)
                self.table.setItem(self.shoot, 11, QTableWidgetItem(str(self.date)))
        
        self.table.selectRow(self.shoot)
        self.Maxx.append(self.maxx)
        self.Minn.append(self.minn)
        self.Summ.append(self.summ)
        self.Mean.append(self.moy)
        self.Xmax.append(self.xmax)
        self.Ymax.append(self.ymax)
        self.Xcmass.append(self.xcmass)
        self.Ycmass.append(self.ycmass)
        self.USER1.append(self.user1)

        self.table.setVerticalHeaderLabels(self.labelsVert)

        #  plot Update plot
        if self.winCoupeMax.isWinOpen is True:
            self.PlotMAX()  # (self.Maxx,axis=self.posMotor,symbol = self.symbol, pen=None,label=self.motor)
        if self.winCoupeMin.isWinOpen is True:
            self.PlotMIN()
        if self.winCoupeXmax.isWinOpen is True:
            self.PlotXMAX()
        if self.winCoupeYmax.isWinOpen is True:
            self.PlotYMAX()
        if self.winCoupeSum.isWinOpen is True:
            self.PlotSUM()
        if self.winCoupeMean.isWinOpen is True:
            self.PlotMEAN()
        if self.winCoupeXcmass.isWinOpen is True:
            self.PlotXCMASS()
        if self.winCoupeYcmass.isWinOpen is True:
            self.PlotYCMASS()
        if self.winCoupeSumThreshold.isWinOpen is True:
            self.PlotSUMTHRESHOLD()
        if self.winCoupeUser1.isWinOpen is True:
            self.PlotUSER1()

        # update zoom windows  
        if self.winZoomMax.isWinOpen is True:
            self.ZoomMAX()
        if self.winZoomSum.isWinOpen is True:
            self.ZoomSUM()
        if self.winZoomMean.isWinOpen is True:
            self.ZoomMEAN() 
        if self.winZoomXmax.isWinOpen is True:
            self.ZoomXmaX()
        if self.winZoomYmax.isWinOpen is True:
            self.ZoomYmAX()
        if self.winZoomCxmax.isWinOpen is True:
            self.ZoomCxmaX()
        if self.winZoomCymax.isWinOpen is True:
            self.ZoomCymAX()   
        if self.winZoomSumThreshold.isWinOpen is True:
            self.ZoomSUMThreshold()
        if self.winZoomUser1.isWinOpen is True:
            self.ZoomUser1()   
        self.shoot += 1
      
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        self.shoot = 0
        self.TableSauv = ['file,Max,Min,x Max,y max,Sum,Mean,Size,x c.mass,y c.mass']
        
        if self.winCoupeMax.isWinOpen is True:
            self.winCoupeMax.close()
        if self.winCoupeMin.isWinOpen is True:
            self.winCoupeMin.close()
        if self.winCoupeXmax.isWinOpen is True:
            self.winCoupeXmax.close()
        if self.winCoupeYmax.isWinOpen is True:
            self.winCoupeYmax.close()
        if self.winCoupeSum.isWinOpen is True:
            self.winCoupeSum.close()
        if self.winCoupeMean.isWinOpen is True:
            self.winCoupeMean.close()
        if self.winCoupeXcmass.isWinOpen is True:
            self.winCoupeXcmass.close()
        if self.winCoupeYcmass.isWinOpen is True:
            self.winCoupeYcmass.close()
        if self.winCoupeSumThreshold.isWinOpen is True:
            self.winCoupeSumThreshold.close()
        time.sleep(0.1)
        event.accept()

    def open_widget(self, fene):
        """ ouverture widget suplementaire
        """

        if fene.isWinOpen is False:
            fene.setup
            fene.isWinOpen = True
            # A=self.geometry()
            # fene.setGeometry(A.left()+A.width(),A.top(),500,A.height())
            fene.show()
        else:
            # fene.activateWindow()
            # fene.raise_()
            fene.showNormal()

    def FctUser1(self):
        # The USER1 function
        a = 0  # 10*random()
        
        return (a)

    def motorChange(self):
        if self.motorNameBox.currentIndex() != 0 :
            self.numMotor = self.motorNameBox.currentIndex()
            
            if self.motA2V is True:
                self.numMotor = self.numMotor -1
                self.MOT = self.A2V.MOTORA2V(self.listMotor[self.numMotor])
                self.stepmotor = self.MOT.step
            if self.motRSAI is True :
                self.IPadress = self.listRack [self.rackChoise.currentIndex()]
                self.MOT = self.RSAI.MOTORRSAI(self.IPadress, self.numMotor)
                self.stepmotor = self.MOT.step
               
            
            self.unit()
        

    def ChangeIPRack(self):
        self.motorNameBox.clear()
        self.IPadress = str( self.listRack[self.rackChoise.currentIndex()])
        #print('ip',self.IPadress)
        self.listMotorName = self.RSAI.listMotorName(self.IPadress)
        #print('listmotorname',self.listMotorName)
        self.rackName = self.RSAI.nameEquipment(self.IPadress)
        self.motorNameBox.addItem('Choose a motor')
        self.motorNameBox.addItems(self.listMotorName)

    def unit(self):
        '''
        unit change mot foc
        '''
        self.indexUnit = self.unitBouton.currentIndex()
        #print('winMeas',self.stepmotor)
        if self.indexUnit == 0:  # step
            self.unitChange = 1
            self.unitName = 'step'
            
        if self.indexUnit == 1:  # micron
            self.unitChange = float((1*self.stepmotor))
            self.unitName = 'um'
        if self.indexUnit == 2:  # mm
            self.unitChange = float((1000*self.stepmotor))
            self.unitName = 'mm'
        if self.indexUnit == 3:  # ps  double passage : 1 microns=6fs
            self.unitChange = float(1*self.stepmotor/0.0066666666)
            self.unitName = ' ps'
        if self.indexUnit == 4:  # en degres
            self.unitChange = 1 * self.stepmotor
            self.unitName = '°'
        
        if self.unitChange == 0:
            self.unitChange = 1  # avoid /0


if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = MEAS(motRSAI=True)
    e.show()
    appli.exec_()


# -*- coding: utf-8 -*-


# !/usr/bin/python
# """andor.py version 1.0
# November 2008
# Michael V. DePalatis <surname at gmail dot com>
#
# Contains functions to handle Andor SIF image files including
# conversion to other formats and reading basic information from the SIF
# file. Little is done in the way of error checking, but this is
# probably not a real issue as long as you actually pass it a valid SIF
# file.
# """


import numpy as np
 

class SifFile:
    """SifFile is the Python representation of an Andor SIF image
    file. Image data is stored in a numpy array indexed as [row,
    column] instead of [x, y]."""

    def __init__(self, path=""):
        self.data = 0
        if path != "":
            self.open(path)
                
    def __add__(self, other):
        new_sif = self.__class__()
        new_sif.__dict__ = self.__dict__
        if isinstance(other, (SifFile, np.ndarray)):
            new_sif.data = new_sif.data + other.data
        elif isinstance(other, (int, float)):
            new_sif.data = new_sif.data + other
        else:
            raise TypeError("Addition of SIF data requires another SifFile instance, a numpy array, or a scalar.")
        return new_sif

    def __sub__(self, other):
        new_sif = self.__class__()
        new_sif.__dict__ = self.__dict__
        if isinstance(other, (SifFile, np.ndarray)):
            new_sif.data = new_sif.data - other.data
        elif isinstance(other, (int, float)):
            new_sif.data = new_sif.data - other
        else:
            raise TypeError("Subtraction of SIF data requires another SifFile instance, a numpy array, or a scalar.")
        return new_sif

    def __mul__(self, other):
        new_sif = self.__class__()
        new_sif.__dict__ = self.__dict__
        if isinstance(other, (SifFile, np.ndarray)):
            new_sif.data = new_sif.data * other.data
        elif isinstance(other, (int, float)):
            new_sif.data = new_sif.data * other
        else:
            raise TypeError("Multiplcation of SIF data requires another SifFile instance, a numpy array, or a scalar.")
        return new_sif

    def __rmul__(self, other):
        return self.__mul__(other)

    def openA(self, path):
        """Opens the SIF file at path and stores the data in
        self.data."""

#        #if path.lower().endswith('.sif'):
        opener = open
#        elif path.lower().endswith('.sif.bz2'): # bz2 compressed sif files
#            import bz2
#            opener = bz2.BZ2File
#        else:
#            raise ValueError('Wrong extension.')

        sif = opener(path, 'rb')

        # Verify we have a SIF file
        if sif.readline().strip() != b"Andor Technology Multi-Channel File":
            sif.close()
            raise Exception("File %s is not an Andor SIF file." % path)

        # Ignore lines until we get to camera model
        for i in range(2):
            sif.readline()

        # Get camera model
        self.cammodel = sif.readline().strip()

        # Get CCD dimension in pixels
        shape = sif.readline().split()
        self.ccd_size = (int(shape[1]), int(shape[0]))
        
        # Read superpixeling data
        line = sif.readline().split()
        # self.shape = (self.ccd_size[1]/int(line[5]), self.ccd_size[0]/int(line[6]))
        for i in range(0):
            line = sif.readline()
            print(line)
        # Read data
        raw_data = sif.read()
        raw_data = raw_data[len(raw_data) - 4*np.prod(self.ccd_size):]
        
        self.data = np.fromstring(raw_data, dtype=np.float32)
        # self.data = self.data[:len(self.data)-2]
        # if line[3] < line[2]:
        #    self.shape = (len(self.data)/int(line[3]), int(line[3]))
        # else:
        #    # I'm not sure if this is correct...
        #    # Needs more testing.
        #    self.shape = (int(line[2]), len(self.data)/int(line[2]))
        
        self.data = np.reshape(self.data, self.ccd_size)
       
        return self.data